#include "libcell.h"

void get_program_list (_comm_header *comm, _path *path, _prog_list *progs, int debug)
{
int x,count;
_data_buffer *buff;
_encaps_header *head;
_data_buffer *cpf_buffer;
_data_buffer *sendbuff;
_data_buffer *msgbuff;
_ioi_data *ioi1;

dprint (DEBUG_TRACE, "get_object_list.c entered.\n");
if (path == NULL)
	{
	printf ("path structure not allocated - get_program_list.c\n");
	return;
	}

if (progs == NULL)
	{
	printf ("tags structure not allocated - get_program_list.c\n");
	return;
	}

buff = malloc(sizeof(_data_buffer));
if (buff == NULL)
	{
	printf ("Could not allocate memory for data buffer - get_program_list.c\n");
	return;
	}
msgbuff = malloc(sizeof(_data_buffer));
if (msgbuff == NULL)
	{
	printf ("Could not allocate memory for msgdata buffer - get_program_list.c\n");
	free (buff);
	return;
	}
head = malloc(sizeof(_encaps_header));
if (head == NULL)
	{
	printf ("Could not allocate memory for Encaps Header - get_program_list.c\n");
	free (buff);
	free (msgbuff);
	return;
	}
cpf_buffer = malloc(sizeof(_data_buffer));
if (cpf_buffer == NULL)
	{
	printf ("Could not allocate memory for cpf_buffer - get_program_list.c\n");
	free(buff);
	free(head);
	free(msgbuff);
	return;
	}
sendbuff = malloc(DATA_Buffer_Length);
if (sendbuff == NULL)
	{
	printf ("Could not allocate memory for send buffer - get_program_list.c\n");
	free(buff);
	free(head);
	free(msgbuff);
	free(cpf_buffer);
	return;
	}
ioi1 = malloc(sizeof(_ioi_data));
if (ioi1 == NULL)
	{
	printf ("Could not allocate memory for ioi1 buffer - get_program_list.c\n");
	free (buff);
	free (head);
	free (msgbuff);
	free (cpf_buffer);
	free (sendbuff);
	return;
	}

progs->count = 0;
while (1)
	{
	memset(buff,0,sizeof(_data_buffer));
	memset(msgbuff,0,sizeof(_data_buffer));
	memset(head,0,sizeof(_encaps_header));
	memset(cpf_buffer,0,sizeof(_data_buffer));
	memset(sendbuff,0,sizeof(_data_buffer));
	memset(ioi1,0,sizeof(_ioi_data));

	buff->len = 0;

	msgbuff->data[msgbuff->len++] = LIST_ALL_OBJECTS;
	ioi1->class = OBJECT_SUB_OBJECT;
	ioi1->instance = BASE_INSTANCE;
	ioi1->member = -1;
	ioi1->point = -1;
	ioi1->attribute = -1;
	ioi1->tagname = NULL;
	ioi(msgbuff, ioi1, NULL ,debug);
	if (debug != DEBUG_NIL)
		{
		dprint (DEBUG_BUILD,"Msgbuff = ");
		for (x=0;x<msgbuff->len;x++)
			dprint (DEBUG_BUILD,"%02X ",msgbuff->data[x]);
		dprint (DEBUG_BUILD,"\n");
		}

	cpf_buffer->data[cpf_buffer->len++] = PDU_Unconnected_Send;
	ioi1->class = CONNECTION_MANAGER;
	ioi1->instance = FIRST_INSTANCE;
	ioi1->member = -1;
	ioi1->point = -1;
	ioi1->attribute = -1;
	ioi1->tagname = NULL;
	
	ioi (cpf_buffer, ioi1, NULL, debug);
	settimeout (6,0x9a,cpf_buffer, debug);
	cpf_buffer->data[cpf_buffer->len++] = msgbuff->len & 255;
	cpf_buffer->data[cpf_buffer->len++] = msgbuff->len / 0x100;

	makepath(path, msgbuff, debug);
	memcpy (cpf_buffer->data + cpf_buffer->len, msgbuff->data, msgbuff->len);
	cpf_buffer->len += msgbuff->len;
	
	sendRRdata (10, comm, head, buff, debug);
	cpf (CPH_Null, NULL, CPH_Unconnected_message, cpf_buffer, buff, debug);

	head->len = buff->len;
	memcpy (sendbuff->data, head, ENCAPS_Header_Length);
	sendbuff->overall_len = ENCAPS_Header_Length;
	if (debug!=DEBUG_NIL)
		{
		dprint (DEBUG_BUILD,"Loading sendbuffer with command data.\n");
		for (x=0;x<buff->len;x++)
			dprint (DEBUG_BUILD,"%02X ",buff->data[x]);
		dprint (DEBUG_BUILD,"\n");
		}
	memcpy (sendbuff->data + sendbuff->overall_len, buff->data, buff->len);
	sendbuff->overall_len += buff->len;
	senddata (sendbuff, comm, debug);
	readdata (buff, comm, debug);
	memset (head,0, ENCAPS_Header_Length);
	memcpy (head,buff->data,ENCAPS_Header_Length);
	if (head->status != 0)
		{
		dprint (DEBUG_VALUES,"Get Program List command returned an error.\n");
		free(ioi1);
		free(buff);
		free(head);
		free(cpf_buffer);
		free(sendbuff);
		free(msgbuff);
		return;
		}
	if (debug != DEBUG_NIL)
		{
		dprint (DEBUG_BUILD,"Got good reply to Get Program List Command - %d\n",buff->overall_len);	
		for (x=44;x < buff->overall_len;x++)
			dprint (DEBUG_BUILD,"%02X ",buff->data[x]);
		dprint (DEBUG_BUILD,"\n");
		}
	count = 0;
	for (x=44; x<buff->overall_len; x += 4)
		{
		progs->prog[progs->count + count] = 0;
		dprint (DEBUG_BUILD,"Allocating memory for program list entry %d\n",progs->count + count);
		progs->prog[progs->count + count] = malloc(sizeof(_prog_detail));
		if (progs->prog[progs->count + count] == NULL)
			{
			printf ("Could not allocate memory for program list entry %d.\n",progs->count + count);
			return;
			}
		dprint (DEBUG_BUILD,"Memory allocated: %p\n",progs->prog[progs->count + count]);
		memset (progs->prog[progs->count + count],0, sizeof(_prog_detail));
		progs->prog[progs->count + count]->base = \
			buff->data[x] + \
			(buff->data[x+1] * 0x100) + \
			(buff->data[x+2] * 0x10000) + \
			(buff->data[x+3] * 0x1000000);
		count++;
		}
	printf ("count = %d\n",count);
	progs->count += count;
	printf ("progs->count = %d\n",progs->count);
	if (buff->data[42] != 6)
		break;
	}
	
free(ioi1);
free(sendbuff);
free(cpf_buffer);
free(head);
free(msgbuff);
free(buff);
dprint(DEBUG_TRACE, "Exiting get_program_list.c\n");
return;
}
